#!/usr/bin/env bash

# -------------------------------- Global variables --------------------------------

DIR="$(dirname "$(readlink -e "${0}")")"
CONFIG_FILE="${DIR}/config"
TESTER_SCRIPT="/root/envsetup/tests/scripts/ms-testing-suite/ms_live_streamer.py"
TIME_DIFF_SCRIPT="/root/envsetup/tools/benchmark/live/resources/time_diff_test.sh"
SYS_INFOS_SCRIPT="/root/envsetup/tools/benchmark/live/resources/sysinfos.sh"
NMON_CMD="nmon -f -s 1 && pgrep nmon"
LOADAVG_CMD="cat /proc/loadavg | awk '{print \"load average:\",\$1\" (1m)\",\$2\" (5m)\",\$3\" (15m)\"}'"
HLS_TMPFS_SIZE_CMD="du -h -d0 /var/tmp/nginx-rtmp/msuser | awk '{print \$1}'"

# -- Script prerequisites tests --

# Configuration loading
if [[ -e "${CONFIG_FILE}" ]]; then
    source "${CONFIG_FILE}"
else
    echo "The configuration file \"${CONFIG_FILE}\" is missing"
    exit 1
fi

# -------------------------------- Functions --------------------------------

# Loading global functions
source "${DIR}/lib/functions.sh"

# Function to cleanup the tests files on script termination
cleanup() {
    msg "white" "Stopping every live streams found on the MediaServer"
    ssh -l root ${MS_IP} -- pkill -f gst-launch
    msg "white" "Purging any remaining playlists on the live server"
    ssh -l root ${LIVE_IP} -- rm -f "${LIVE_HLS_FOLDER}/"*
    msg "white" "Restarting nginx on the live server"
    ssh -l root ${LIVE_IP} -- systemctl restart nginx
}

cleanup_trap() {
    cleanup

    exit 1
}

# Launch X streams during 30s on the MediaServer (with X being the number of streams to spawn as argument)
launch_stream() {
    nb_streams="${1}"
    timeout_tester="${2}"

    read -r pid < <(ssh -l root ${MS_IP} -- "${TESTER_SCRIPT} --num-streamers ${nb_streams} ${MS_INSTANCE} &>/dev/null & echo \$!")

    echo ${pid}
}

usage() {
    cat << EOF
Usage: ./$(basename "${0}") [OPTS] STREAM_NUMBER

OPTIONS:
   -t --test-duration    INT  test duration in seconds (default: 30)
   -l --log-file         PATH path for the log file (by default a log is created in the script "output" dir)
   -n --no-header             omit the servers hardware informations
   -m --nmon-monitoring       launch nmon on Mediaserver and Live server
   -h --help                  show this help
EOF
}

# -------------------------------- Parameters --------------------------------

if [[ $# -eq 0 ]]; then
    usage

    exit 0
fi

TEST_DURATION=30
NO_HEADER=false
NMON=false

while [[ $# -ne 0 ]]; do
    case "${1}" in
        -t|--test-duration)    shift; TEST_DURATION=${1};;
        -l|--log-file)         shift; A_LOG_FILE=${1};;
        -n|--no-header)        NO_HEADER=true;;
        -n|--nmon-monitoring)  NMON=true;;
        -h|--help)             usage; exit 0;;
        --)                    shift; break;;
        -*)                    echo "unknown ${1} option"; exit 1;;
        *)                     STREAM_NUMBER=${1};;
    esac
    shift
done

if [[ ! "${STREAM_NUMBER}" =~ ^[0-9]+$ ]]; then
    usage
    exit 1
fi

# -------------------------------- Main --------------------------------

# The trap is played on associated signals calls to cleanup bench files
trap cleanup_trap SIGTERM SIGINT

# Before launching the bench we check that everything is setup correctly
# We check that the live chunks are generated in a tmpfs
hls_fs_type="$(ssh -l root ${LIVE_IP} -- findmnt ${LIVE_HLS_FOLDER} -o FSTYPE | tail -n1)"

if [[ "${hls_fs_type}" != "tmpfs" ]]; then
    echo -e "${RED}On the live server, the chunks folder \"${LIVE_HLS_FOLDER}\" is not mounted on a tmpfs as required${NC}"

    exit 1
fi

# We check that the RTMP module default limit has been overwritten
if ! ssh -l root ${LIVE_IP} -- "grep -qR 'max_streams 1000;' /etc/nginx/{global-conf.d,rtmp.d}"; then
    echo -e "${RED}The rtmp \"max_streams\" limit should be set to 1000 for the bench${NC}"
    echo -e "${RED}On the live server, add the \"max_streams 1000;\" parameter in the \"rtmp\" block of the \"/etc/nginx/global-conf.d/rtmp.conf\" configuration file under the \"chunk_size\" parameter.Then restart nginx${NC}"

    exit 1
fi

# Preparing the output reports variables
OUTPUT_DIR="${DIR}/output/$(date +'%Y%m%d%H%M')-$(basename "${0%%.sh}")"
if [[ -z ${A_LOG_FILE} ]]; then
    LOG_FILE="${OUTPUT_DIR}/bench.log"
else
    if [[ -e $(dirname ${A_LOG_FILE}) ]]; then
        LOG_FILE="${A_LOG_FILE}"
    else
        msg "red" "The directory for the log file does not exist"
        exit 1
    fi
fi

NMON_FILE="${OUTPUT_DIR}/bench.nmon"

if [[ -z ${A_LOG_FILE} ]] || ${NMON} || ! ${NO_HEADER}; then
    # Creation of the logs directory
    if [[ ! -e "${OUTPUT_DIR}" ]]; then
        mkdir -p "${OUTPUT_DIR}"
    fi
fi

# We print the configuration (CPU, RAM) of the benched servers
if ! ${NO_HEADER}; then
    ${DIR}/resources/sysinfos.sh ${OUTPUT_DIR}/sysinfos.txt ${LOG_FILE}
fi

# Test variables
failure=false

# Print load average and tmpfs size
LOAD_MS="$(ssh -l root ${MS_IP} -- "${LOADAVG_CMD}")"
LOAD_LIVE="$(ssh -l root ${LIVE_IP} -- "${LOADAVG_CMD}")"
SIZE_LIVE="$(ssh -l root ${LIVE_IP} -- "${HLS_TMPFS_SIZE_CMD}")"

msg "green" "MediaServer ${LOAD_MS}"
msg "green" "Live server ${LOAD_LIVE}"
msg "green" "Live server ${LIVE_HLS_FOLDER} size: ${SIZE_LIVE}"

if ${NMON}; then
    # We launch the dstat command on the Live server to collect the system performance datas
    msg "white" "Launching the system performances data collection on the Live server"
    #ssh -l root ${LIVE_IP} -- "[[ -e /tmp/bench.csv ]] && rm /tmp/bench.csv"
    ssh -l root ${LIVE_IP} -- 'rm -f /tmp/*.nmon'
    nmon_pid=$(ssh -l root ${LIVE_IP} -- "cd /tmp && ${NMON_CMD}")
fi

# Launch X streams (X being the current step value)
msg "blue" "MediaServer - Launching the tests with ${STREAM_NUMBER} streams..."
tester_pid=$(launch_stream ${STREAM_NUMBER})

# Waiting for the streams to spawn correctly
msg "white" "Waiting for streams to spawn on the mediaserver"
wanted_playlist_nb=$(( 1 * STREAM_NUMBER ))
real_playlist_nb=$(ssh -l root ${LIVE_IP} -- ls -1 "${LIVE_HLS_FOLDER}/"*.m3u8 2>/dev/null | wc -l)

while [[ ${real_playlist_nb} -ne ${wanted_playlist_nb} ]]; do
    sleep 1
    real_playlist_nb=$(ssh -l root ${LIVE_IP} -- ls -1 "${LIVE_HLS_FOLDER}/"*.m3u8 2>/dev/null | wc -l)
done

# We wait a bit for the streams to start
sleep 10

# Launching the checks on the live server
msg "blue" "Live server - Checking the playlists refreshing times every seconds for ${TEST_DURATION} checks (~1s per check)"
ret=$(ssh -l root ${LIVE_IP} -- ${TIME_DIFF_SCRIPT} "${LIVE_HLS_FOLDER}" ${TEST_DURATION} ${wanted_playlist_nb})

if [[ "$?" == "1" ]]; then
    failure=true
fi

if [[ "${ret}" != "" ]]; then
    msg "orange" "${ret}"
fi

# Print load average and tmpfs size
LOAD_MS="$(ssh -l root ${MS_IP} -- "${LOADAVG_CMD}")"
LOAD_LIVE="$(ssh -l root ${LIVE_IP} -- "${LOADAVG_CMD}")"
SIZE_LIVE="$(ssh -l root ${LIVE_IP} -- "${HLS_TMPFS_SIZE_CMD}")"

msg "green" "MediaServer ${LOAD_MS}"
msg "green" "Live server ${LOAD_LIVE}"
msg "green" "Live server ${LIVE_HLS_FOLDER} size: ${SIZE_LIVE}"

# We stop the tester
msg "white" "Stopping the MediaServer test streams and waiting for them to despawn"
ssh -l root ${MS_IP} -- "kill -s SIGINT ${tester_pid}"

# We stop nmon on the Live server and we retrieve the resulting CSV
if ${NMON}; then
    msg "white" "Stopping the collect of the system performance datas on the Live server"
    ssh -l root ${LIVE_IP} -- "kill ${nmon_pid}"
    msg "white" "Retrieving the nmon report from the Live server"
    last_nmon="$(ssh -l root ${LIVE_IP} -- 'ls -1t /tmp/*.nmon | head -n1')"
    scp "root@${LIVE_IP}:${last_nmon}" "${NMON_FILE}"

    # Generation of the HTML Graphs page
    ${DIR}/lib/nmonchart ${NMON_FILE} ${OUTPUT_DIR}/graph.html
fi

# Print the output files path
if ${NMON}; then
    msg "white" "Bench NMON file:  ./$(realpath --relative-to ${DIR} ${NMON_FILE})"
    msg "white" "Bench graph page: ./$(realpath --relative-to ${DIR} ${OUTPUT_DIR}/graph.html)"
fi
msg "white" "Bench logs:       ./$(realpath --relative-to ${DIR} ${LOG_FILE})"

sleep 2

# We purge the playlists files on the live server and we restart nginx to be in an initial case
cleanup

if ${failure}; then
    msg "red" "Anomalies has been detected"
    exit 1
else
    msg "green" "No anomalies detected"
    exit 0
fi
