#!/usr/bin/env bash

DIR="$(dirname "$(readlink -e "${0}")")"
CONFIG_FILE="${DIR}/config"
OUTPUT_DIR="${DIR}/output/$(date +"%Y%m%d%H%M")-$(basename "${0%%.sh}")"
LOG_FILE="${OUTPUT_DIR}/bench.log"

# Loading global functions
source "${DIR}/lib/functions.sh"

# Print usage function
usage() {
    cat << EOF
Usage: ./$(basename "${0}") [OPTS] -- STREAMS_TARGETS1 STREAMS_TARGET2 ...

OPTIONS:
   -t --test-duration  INT  test duration in seconds (default: 300)
   -h --help                show this help

STREAMS_TARGETS is a list (space separated), consisting of numbers of streams you want to benchmark.
For example, if you want to benchmark the solution for 100, 200, 300 and 400 streams:
./$(basename "${0}") -- 100 200 300 400

You can also use bash string extention, like this
./$(basename "${0}") -- {1,2,3,4}00
EOF
}

cleanup() {
    # Kill the running nmon monitors
    ssh -l root ${MS_IP}   -- "pkill -f nmon"
    ssh -l root ${LIVE_IP} -- "pkill -f nmon"

    # We retrieve the nmon metrics DB
    scp -q root@mstest:/tmp/*.nmon ${OUTPUT_DIR}/bench-ms.nmon
    scp -q root@livetest:/tmp/*.nmon ${OUTPUT_DIR}/bench-live.nmon

    # Clean the metrics DB on the remotes
    ssh -l root ${MS_IP}   -- "rm -f /tmp/*.nmon"
    ssh -l root ${LIVE_IP} -- "rm -f /tmp/*.nmon"

    # Create graphs from the metrics
    ${DIR}/lib/nmonchart ${OUTPUT_DIR}/bench-ms.nmon   ${OUTPUT_DIR}/graph-ms.html
    ${DIR}/lib/nmonchart ${OUTPUT_DIR}/bench-live.nmon ${OUTPUT_DIR}/graph-live.html
}

cleanup_trap() {
    cleanup

    exit 1
}

trap cleanup_trap SIGINT

# Configuration loading
if [[ -e "${CONFIG_FILE}" ]]; then
    source "${CONFIG_FILE}"
else
    echo "The configuration file \"${CONFIG_FILE}\" is missing"
    exit 1
fi

TEST_DURATION=300

# Arguments parsing
while [[ $# -ne 0 ]]; do
    case "${1}" in
        -t|--test-duration)    shift; TEST_DURATION=${1};;
        -h|--help)             usage; exit 0;;
        --)                    shift; break;;
        -*)                    echo "unknown ${1} option"; exit 1;;
        *)                     break;;
    esac
    shift
done

STREAMS_TARGETS="$@"

if [[ ! ${STREAMS_TARGETS} =~ ^[0-9]+([[:blank:]][0-9]+)*$ ]]; then
    usage
    exit 1
fi

# Creating the output dir
[[ ! -e ${OUTPUT_DIR} ]] && mkdir -p ${OUTPUT_DIR}

# Getting the system informations
${DIR}/resources/sysinfos.sh ${OUTPUT_DIR}/sysinfos.txt ${LOG_FILE}

# Clean previous benchmarks metrics
ssh -l root ${MS_IP}   -- "rm -f /tmp/*.nmon"
ssh -l root ${LIVE_IP} -- "rm -f /tmp/*.nmon"

# Start new metrics generation with nmon
ssh -l root ${MS_IP}   -- "cd /tmp && nmon -f -s 1 -c 9999999"
ssh -l root ${LIVE_IP} -- "cd /tmp && nmon -f -s 1 -c 9999999"

# Launch a bench for each provided stream number targets
for stream_target in ${STREAMS_TARGETS}; do
    msg "orange" "Begining of the bench for ${stream_target} streams"
    ./bench-live.sh --log-file ${LOG_FILE} --no-header --test-duration ${TEST_DURATION} ${stream_target}

    # If there are more than 10 anomalies in the current test => stop the bench
    anomalies_nb=$(grep -F 'anomalie(s) has been found during the test' ${LOG_FILE} | \
                       grep -oP '[0-9]+(?= anomalie\(s\))' | \
                       sort | \
                       tail -n1)
    if [[ ${anomalies_nb} -gt 10 ]]; then
        msg "red" "More than 10 anomalies has been found with the current streams number"
        msg "red" "Stopping the bench"
        break
    fi

    # If it is not the last bench
    if [[ "${stream_target}" != "${STREAMS_TARGETS##* }" ]]; then
        msg "orange" "We wait 5min before the next bench (to stabilize the load average)"
        sleep 300
    fi
done

cleanup

exit 0
